#define ORBITER_MODULE


#include "TD_Pts_Updater.h"
#include <algorithm>
#include "Orbitersdk.h"


using namespace std;
//TD_Pts_Updater* LinkMgr;

DLLCLBK void InitModule(HINSTANCE hDLL) {
	
	oapiRegisterModule(new TD_Pts_Updater(hDLL));
	
	//g_hInst = hDLL;
	//g_dwCmd = oapiRegisterCustomCmd("Link Manager","Plugin for Link type connections",)
}

DLLCLBK void ExitModule(HINSTANCE hModule) {
	
}




TD_Pts_Updater::TD_Pts_Updater(HINSTANCE hDLL) : oapi::Module(hDLL) {
	TD_def = TD_Pts_Definitions();
	oapiWriteLogV("TouchDown Points Updater Loaded, Version:%i", VERSION);
	frame_counter = 0;
	first_verify = true;
}

TD_Pts_Updater::~TD_Pts_Updater() {}

bool TD_Pts_Updater::ReadConfig(string name) {
	string fn("\\TD_Pts_Updater\\");
	fn += name;
	string unders = "___";
	string slash = "/";
	size_t pos = fn.find(slash);
	while (pos != string::npos) {
		fn.replace(pos, slash.size(), unders);
		pos = fn.find(slash);
	}
	
	fn += ".cfg";
	FILEHANDLE file_h = oapiOpenFile(fn.c_str(), FILE_IN_ZEROONFAIL, CONFIG);
	if (file_h == NULL) {
		oapiWriteLogV("TD_Pts_Updater: No TD Pts Override for: %s", name.c_str());
		return false;
	}
	TD_def = TD_Pts_Definitions();

	int n_points = 0;
	oapiReadItem_int(file_h, "N", n_points);
	if (n_points < 3) {
		return false;
	}
	TD_def.n_points = n_points;
	char cbuf[256] = { '\0' };
	char readbuf[256] = { '0' };
	for (UINT i = 1; i <= n_points; i++) {
		sprintf(cbuf, "POINT_%i", i);
		oapiReadItem_string(file_h, cbuf, readbuf);
		double values[7] = { 0 };
		sscanf(readbuf, "%lf%lf%lf%lf%lf%lf%lf", &values[0], &values[1], &values[2], &values[3], &values[4], &values[5],&values[6]);
		TOUCHDOWNVTX tdvtx;
		tdvtx.pos.x = values[0];
		tdvtx.pos.y = values[1];
		tdvtx.pos.z = values[2];
		tdvtx.stiffness = values[3];
		tdvtx.damping = values[4];
		tdvtx.mu = values[5];
		if (i <= 3) {
			tdvtx.mu_lng = values[6];
		}
		else {
			tdvtx.mu_lng = values[5];
		}
			TD_def.tdvtx.push_back(tdvtx);
		
		
	}

	return true;
}
void TD_Pts_Updater::VerifyVessel(OBJHANDLE hVessel) {
	char name[256] = { '\0' };
	oapiGetObjectName(hVessel, name, 256);
	string str_name(name);
	if (ReadConfig(str_name)) {
		oapiWriteLogV("TD Points Updater: Vessel Definition found for name %s", str_name.c_str());
		ApplyTouchdownPoints(hVessel);
	}
	else {
		if (oapiIsVessel(hVessel)) {
			VESSEL *v = oapiGetVesselInterface(hVessel);
			sprintf(name, v->GetClassName());
			str_name.assign(name);
			if (ReadConfig(str_name)) {
				oapiWriteLogV("TD Points Updater: Vessel Definition found for class %s", str_name.c_str());
				ApplyTouchdownPoints(hVessel);
			}
		}
		
	}
	
	return;
}
void TD_Pts_Updater::clbkSimulationStart(RenderMode mode) {
//	for (DWORD i = 0; i < oapiGetVesselCount(); i++) {
//		OBJHANDLE hVessel = oapiGetObjectByIndex(i);
//		VerifyVessel(hVessel);
//	}
	return;
}
void TD_Pts_Updater::clbkPostStep(double simt, double simdt, double mjd) {
	if (first_verify) {
		frame_counter++;
		if (frame_counter == 10) {
			for (DWORD i = 0; i < oapiGetVesselCount(); i++) {
				OBJHANDLE hVessel = oapiGetVesselByIndex(i);
				VerifyVessel(hVessel);
			}
			first_verify = false;
		}
	}
	
	
}
void TD_Pts_Updater::clbkNewVessel(OBJHANDLE hVessel) {
	VerifyVessel(hVessel);
	return;
}

void TD_Pts_Updater::ApplyTouchdownPoints(OBJHANDLE hVessel) {
	double stiffness = 0;
	double damping = 0;
	if ((TD_def.tdvtx[0].damping == 0) || (TD_def.tdvtx[0].stiffness == 0)) {
		double mass = oapiGetMass(hVessel);
		double x_target = -1;
		stiffness = (-1)*(mass*9.81) / x_target;
		damping = 0.95 * (2 * sqrt(mass*stiffness));
		for (UINT i = 0; i < TD_def.tdvtx.size(); i++) {
			TD_def.tdvtx[i].damping = damping;
			TD_def.tdvtx[i].stiffness = stiffness;
		}
	}
	if (TD_def.tdvtx[0].mu == 0) {
		for (UINT k = 0; k < TD_def.tdvtx.size(); k++) {
			TD_def.tdvtx[k].mu = 3.0;
			TD_def.tdvtx[k].mu_lng = 3.0;
		}
	}

	TOUCHDOWNVTX *td_vtx = new TOUCHDOWNVTX[TD_def.n_points];
	for (UINT j = 0; j < TD_def.n_points; j++) {
		td_vtx[j] = TD_def.tdvtx[j];
	}
	VESSEL *v = oapiGetVesselInterface(hVessel);
	v->SetTouchdownPoints(td_vtx, TD_def.n_points);
	for (UINT q = 0; q < TD_def.n_points; q++) {
		oapiWriteLogV("point%i : %.3f %.3f %.3f %.3f %.3f %.3f %.3f", q + 1, td_vtx[q].pos.x, td_vtx[q].pos.y, td_vtx[q].pos.z, td_vtx[q].stiffness, td_vtx[q].damping, td_vtx[q].mu, td_vtx[q].mu_lng);
	}
	delete td_vtx;

	return;
}

DLLCLBK void opcSaveState(FILEHANDLE scn) {
	
}

DLLCLBK void opcLoadState(FILEHANDLE scn) {
	
}